/*
 * main.c: command line parsing and top level
 */

#include <stdio.h>
#include <stdlib.h>
#include "skel.h"
#include "coopt.h"

int main(int argc, char const * const * argv) {
    int nogo;
    int errs;
    struct coopt_option option[] = {
      { 'h', COOPT_NO_PARAM, "help" },
      { 'l', COOPT_NO_PARAM, "license" },
      { 'l', COOPT_NO_PARAM, "licence" },
      { 'v', COOPT_NO_PARAM, "version" },
      { 'o', COOPT_REQUIRED_PARAM, "output" }
    };
    struct coopt_state state;
    struct coopt_return ret;

    coopt_init(&state, option, sizeof(option) / sizeof(struct coopt_option),
               argc-1, argv+1);

    /*
     * Set up initial (default) parameters.
     */
    nogo = errs = FALSE;

    /*
     * Parse command line arguments.
     */
    do {
      ret = coopt(&state);
      if (!coopt_is_error(ret.result))
      {
        if (ret.opt!=NULL)
        {
          switch (ret.opt->short_option)
          {
            case 'h':
              help();
              nogo=TRUE;
              break;
            case 'v':
              showversion();
              nogo=TRUE;
              break;
            case 'l':
              licence();
              nogo=TRUE;
              break;
            case 'o':
              /* parameter to option is in ret.param */
              break;
          }
        }
        else
        {
          /* non-option argument */
        }
      }
    } while (coopt_is_okay(ret.result));

    if (coopt_is_error(ret.result)) {
        char temp[256]; /* must be large enough to take longest option name */
        switch (ret.result)
        {
          case COOPT_RESULT_NOPARAM:
          case COOPT_RESULT_MISSINGPARAM:
            coopt_sopt(temp, 256, &ret, 0, &state);
            error(err_optnoarg, temp);
            break;
          case COOPT_RESULT_BADOPTION:
            coopt_sopt(temp, 256, &ret, 0, &state);
            error(err_nosuchopt, temp);
            break;
          default:
            /* FIXME: ought to say "a problem here"
             * Possibles: MULTIMIXED, HADPARAM, AMBIGUOUSOPT
             * Only HADPARAM might occur currently (--version=6).
             * Also: ERROR (coopt internal error), but this should never happen.
             */
            break;
        }
	exit(EXIT_FAILURE);
    }
    if (nogo)
	exit(EXIT_SUCCESS);

    /*
     * Do the work.
     */
    printf("FIXME: put some activity here\n");

    return 0;
}
