#!/bin/sh

FILES='Makefile error.c help.c licence.c malloc.c skel.h version.c'
ALTFILES='coopt-main.c main.c'

# Find ourself

echo 'This is Skel, the Tartarus application wizard'
echo ''
echo -n 'Searching for myself...'

SELF=
test -f $0 && SELF=$0
test -z "$SELF" -a -f ./newapp.sh && SELF=./newapp.sh
test -z "$SELF" -a -f skel/newapp.sh && SELF=skel/newapp.sh

if test -z "$SELF"; then
  echo 'can'\''t find newapp.sh, giving up'
  exit 1
fi

echo "found at \`$SELF'"

DIR=`echo $SELF | sed 's:/[^/]*$::'`
test -z $DIR && DIR=.

if test -d $DIR; then
  echo -n "Looking for my files in directory \`$DIR'..."
else
  echo 'Can'\''t find myself in a directory, giving up'
  exit 1
fi

for file in $FILES $ALTFILES; do
  if test -f $DIR/$file; then
    : well and good
  else
    echo "can't find \`$DIR/$file', giving up"
    exit 1
  fi
done

echo 'found everything'
echo ''

# Application name

if test $# -gt 0; then
  APPNAME=$1
  shift
else
  echo -n 'Please enter the name of your new project: '
  read APPNAME
fi

appname=`echo $APPNAME | tr A-Z a-z`
appNAME=`echo $appname | tr a-z A-Z`
appN=`echo -n $appNAME | sed 's/^\(.\).*$/\1/'`
appame=`echo -n $appname | sed 's/^.//'`
appName="$appN$appame"

# Application directory

if test $# -gt 0; then
  APPDIR=$1
  shift
else
  # FIXME: invent a plausible first guess
  echo -n 'Please enter a path name for your new project'\''s files: '
  read APPDIR
fi

if test -d $APPDIR; then
  echo -n 'This directory already exists. Continue? [yn] '
  read yesno
  if test "$yesno" != "y" && test "$yesno" != "yes" &&
     test "$yesno" != "Y" && test "$yesno" != "YES"; then
    echo ''
    echo 'New application creation cancelled.'
    exit 1
  fi
else
  mkdir -p $APPDIR >&/dev/null || mkdir $APPDIR
fi

echo -n 'Do you want coopt support (alternative is hand-written options parsing)? [Yn] '
read yesno
if test "$yesno" != "y" && test "$yesno" != "yes" &&
   test "$yesno" != "Y" && test "$yesno" != "YES" && test "$yesno" != ""; then
     whichmain="main.c"
else
   whichmain="coopt-main.c"
   echo 'Assuming that:'
   echo ' * libcoopt is installed correctly'
   echo ' * coopt.h is in /usr/include/libcoopt/'
   echo ''
   echo 'If not, you'\''ll have to edit Makefile yourself.'
fi

for file in $FILES; do
  if test "$file" = "skel.h"; then
    new=${APPNAME}.h
  else
    new=$file
  fi
  echo -n "Creating $APPDIR/$new..."
  if test "$file" = "Makefile"; then
    if test "$whichmain" = "coopt-main.c"; then
      sed -e "s/skel/$appname/g" \
          -e "s/Skel/$appName/g" \
          -e "s/SKEL/$appNAME/g" \
          -e "s%#COOPT_CFLAGS%CFLAGS += -I/usr/include/libcoopt/%" \
          -e "s/#COOPT_LIBS/LIBS += -lcoopt/" \
          < $DIR/$file > $APPDIR/$new
    else
      sed -e "s/skel/$appname/g" \
          -e "s/Skel/$appName/g" \
          -e "s/SKEL/$appNAME/g" \
          -e "s/#COOPT_LIBS//" \
          -e "s/#COOPT_CFLAGS//" \
          < $DIR/$file > $APPDIR/$new
    fi
  else
    sed -e "s/skel/$appname/g" \
        -e "s/Skel/$appName/g" \
        -e "s/SKEL/$appNAME/g" \
        < $DIR/$file > $APPDIR/$new
  fi
  echo 'done'
done

echo -n "Creating $APPDIR/main.c..."
sed -e "s/skel/$appname/g" \
    -e "s/Skel/$appName/g" \
    -e "s/SKEL/$appNAME/g" \
    < $DIR/$whichmain > $APPDIR/main.c
echo 'done'
